@@ -1,7 +1,7 @@
 use crate::data::{
-    ast::ForgetMemory, primitive::PrimitiveNull,
-    hold::Hold, message::Message, Literal, Memory, MessageData,
-    error_info::ErrorInfo
+    ast::ForgetMemory, error_info::ErrorInfo, hold::Hold, message::Message,
+    primitive::PrimitiveNull, Literal, Memory, MessageData,
+    csml_logs::LogLvl,
 };
 
 use std::sync::mpsc;
@@ -15,6 +15,12 @@ pub enum MSG {
     Remember(Memory),
     Forget(ForgetMemory),
     Message(Message),
+    Log{
+        flow: String,
+        line: u32,
+        message: String,
+        log_lvl: LogLvl,
+    },
     Hold(Hold),
     Next {
         flow: Option<String>,
@@ -52,7 +58,10 @@ impl MSG {
                     sender.send(msg).unwrap();
                 }
 
-                PrimitiveNull::get_literal(err.position.interval)
+                let mut error_lit = PrimitiveNull::get_literal(err.position.interval);
+                error_lit.additional_info = err.additional_info;
+
+                error_lit
             }
         }
     }
