@@ -83,6 +83,19 @@ pub fn match_actions(
             MSG::send(&sender, MSG::Message(msg.clone()));
             Ok(Message::add_to_message(msg_data, MessageType::Msg(msg)))
         }
+        ObjectType::Log{expr, interval, log_lvl} => {
+            let args = resolve_fn_args(expr, data, &mut msg_data, sender)?;
+            let log_msg = args.args_to_log();
+
+            MSG::send(&sender, MSG::Log{
+                flow: data.context.flow.to_owned(),
+                line: interval.start_line,
+                message: log_msg,
+                log_lvl: log_lvl.to_owned(),
+            });
+
+            Ok(msg_data)
+        }
         ObjectType::Use(arg) => {
             expr_to_literal(arg, false, None, data, &mut msg_data, sender)?;
             Ok(msg_data)
@@ -95,6 +108,7 @@ pub fn match_actions(
             let (
                 tmp_flows,
                 tmp_flow,
+                tmp_default_flow,
                 mut tmp_context,
                 tmp_event,
                 tmp_env,
@@ -109,6 +123,7 @@ pub fn match_actions(
             let mut new_scope_data = Data::new(
                 &tmp_flows,
                 &tmp_flow,
+                tmp_default_flow,
                 &mut tmp_context,
                 &tmp_event,
                 &tmp_env,
@@ -138,6 +153,7 @@ pub fn match_actions(
                             new_value = Literal {
                                 content_type: new_value.content_type,
                                 interval: new_value.interval,
+                                additional_info: None,
                                 primitive
                             };
                         }
@@ -154,6 +170,7 @@ pub fn match_actions(
                             new_value = Literal {
                                 content_type: new_value.content_type,
                                 interval: new_value.interval,
+                                additional_info: None,
                                 primitive
                             };
                         }
